@@ -7,12 +7,13 @@ import sys
 from copy import deepcopy
 from datetime import datetime
 from os.path import join
-from typing import List, Union, Optional, Dict, Any
+from typing import List, Union, Optional, Dict, Any, Tuple, Callable, Iterator, Iterable, Set
 
 from hdx.location.country import Country
 from hdx.utilities import is_valid_uuid
 from hdx.utilities.dateparse import parse_date_range, parse_date
-from hdx.utilities.dictandlist import merge_two_dictionaries
+from hdx.utilities.dictandlist import merge_two_dictionaries, write_list_to_csv, DictUpperBound
+from hdx.utilities.downloader import Download
 from six.moves import range
 
 import hdx.data.filestore_helper
@@ -869,31 +870,40 @@ class Dataset(HDXObject):
             self.set_dataset_date_from_datetime(date, enddate)
 
     def set_dataset_year_range(self, dataset_year, dataset_end_year=None):
-        # type: (Union[str, int], Optional[Union[str, int]]) -> None
+        # type: (Union[str, int, Iterable], Optional[Union[str, int]]) -> List[int]
         """Set dataset date as a range from year or start and end year.
 
         Args:
-            dataset_year (Union[str, int]): Dataset year given as string or int
+            dataset_year (Union[str, int, Iterable]): Dataset year given as string or int or range in an iterable
             dataset_end_year (Optional[Union[str, int]]): Dataset end year given as string or int
 
         Returns:
-            None
+            List[int]: The start and end year if supplied or sorted list of years
         """
+        retval = list()
+        if isinstance(dataset_year, str):
+            dataset_year = int(dataset_year)
         if isinstance(dataset_year, int):
             dataset_date = '01/01/%d' % dataset_year
-        elif isinstance(dataset_year, str):
-            dataset_date = '01/01/%s' % dataset_year
+            retval.append(dataset_year)
+        elif isinstance(dataset_year, Iterable):
+            retval = sorted(list(dataset_year))
+            dataset_date = '01/01/%d' % retval[0]
+            dataset_end_year = retval[-1]
+            retval.pop()
         else:
             raise hdx.data.hdxobject.HDXError('dataset_year has type %s which is not supported!' % type(dataset_year).__name__)
+        if isinstance(dataset_end_year, str):
+            dataset_end_year = int(dataset_end_year)
         if dataset_end_year is None:
-            dataset_end_year = dataset_year
-        if isinstance(dataset_end_year, int):
+            dataset_end_date = '31/12/%d' % dataset_year
+        elif isinstance(dataset_end_year, int):
             dataset_end_date = '31/12/%d' % dataset_end_year
-        elif isinstance(dataset_end_year, str):
-            dataset_end_date = '31/12/%s' % dataset_end_year
+            retval.append(dataset_end_year)
         else:
             raise hdx.data.hdxobject.HDXError('dataset_end_year has type %s which is not supported!' % type(dataset_end_year).__name__)
         self.set_dataset_date(dataset_date, dataset_end_date)
+        return retval
 
     @classmethod
     def list_valid_update_frequencies(cls):
@@ -1575,24 +1585,203 @@ class Dataset(HDXObject):
         return '%s/dataset/%s' % (self.configuration.get_hdx_site_url(), name)
 
     def remove_dates_from_title(self, change_title=True, set_dataset_date=False):
-        # type: (bool, bool) -> str
-        """Remove dates from dataset title returning True if dates were found in title or False if not. The
-        title in the dataset metadata will be changed by default. The dataset's metadata field dataset date will
-        not be changed by default.
+        # type: (bool, bool) -> List[Tuple[datetime,datetime]]
+        """Remove dates from dataset title returning sorted the dates that were found in title. The title in
+        the dataset metadata will be changed by default. The dataset's metadata field dataset date will not
+        be changed by default, but if set_dataset_date is True, then the range with the lowest start date
+        will be used to set the dataset date field.
 
         Args:
             change_title (bool): Whether to change the dataset title. Defaults to True.
-            set_dataset_date (bool): Whether to set the dataset date from date(s) in the title. Defaults to False.
+            set_dataset_date (bool): Whether to set the dataset date from date or range in the title. Defaults to False.
 
         Returns:
-            bool: True if dates were found in title, False if not
+            List[Tuple[datetime,datetime]]: Date ranges found in title
         """
         if 'title' not in self.data:
             raise HDXError('Dataset has no title!')
         title = self.data['title']
-        newtitle, startdate, enddate = DatasetTitleHelper.get_date_from_title(title)
+        newtitle, ranges = DatasetTitleHelper.get_dates_from_title(title)
         if change_title:
             self.data['title'] = newtitle
-        if set_dataset_date and startdate:
+        if set_dataset_date and len(ranges) != 0:
+            startdate, enddate = ranges[0]
             self.set_dataset_date_from_datetime(startdate, enddate)
-        return newtitle != title
+        return ranges
+
+    def generate_resource_from_rows(self, folder, filename, rows, resourcedata, headers=None):
+        # type: (str, str, List[Union[DictUpperBound, List]], Dict, Optional[List[str]]) -> hdx.data.resource.Resource
+        """Write rows to csv and create resource, adding to it the dataset
+
+        Args:
+            folder (str): Folder to which to write file containing rows
+            filename (str): Filename of file to write rows
+            rows (List[Union[DictUpperBound, List]]): List of rows
+            resourcedata (Dict): Resource data
+            headers (Optional[List[str]]): List of headers. Defaults to None.
+
+        Returns:
+            hdx.data.resource.Resource: The created resource
+        """
+        filepath = join(folder, filename)
+        write_list_to_csv(filepath, rows, headers=headers)
+        resource = hdx.data.resource.Resource(resourcedata)
+        resource.set_file_type('csv')
+        resource.set_file_to_upload(filepath)
+        self.add_update_resource(resource)
+        return resource
+
+    def generate_resource_from_download(self, headers, iterator, hxltags, folder, filename, resourcedata,
+                                        yearcol=None, year_function=None, quickcharts=None):
+        # type: (List[str], Iterator[Union[List,Dict]], Dict[str,str], str, str, Dict, Optional[Union[int,str]], Optional[Callable[[Set[int],Dict],Optional[bool]]], Optional[Dict]) -> Tuple[bool, Dict]
+        """Given headers and an iterator, write rows to csv and create resource, adding to it the dataset. The returned
+        dictionary will contain the headers in the key headers and the list of rows in the key rows.
+
+        The date of dataset can optionally be set by supplying a column in which the year is to be looked up or
+        a function to obtain the year from a row. The function should return False or None to keep the row or True
+        to ignore it. In either case, the list of years is returned in the key years of the returned dictionary.
+
+        If the parameter quickcharts is supplied then various QuickCharts related actions will occur depending upon the
+        keys given in the dictionary. If the keys: hashtag - the HXL hashtag to examine - and values - the 3 values to
+        look for in that column - are supplied, then a list of booleans indicating which QuickCharts bites should be
+        enabled will be returned in the key bites_disabled in the returned dictionary. If the key: cutdown is given,
+        if it is 1, then a separate cut down list is created containing only columns with HXL hashtags and
+        rows with desired values (if hashtag and values are supplied) for the purpose of driving QuickCharts.
+        It is returned in the key qcrows in the returned dictionary with the matching headers in qcheaders.
+        If cutdown is 2, then a resource is created using the cut down list. If the key cutdownhashtags is supplied,
+        then only the provided hashtags are used for cutting down otherwise the full list of hxl tags is used.
+
+        Args:
+            headers (List[str]): Headers
+            iterator (Iterator[Union[List,Dict]]): Iterator returning rows
+            hxltags (Dict[str,str]): Header to HXL hashtag mapping
+            folder (str): Folder to which to write file containing rows
+            filename (str): Filename of file to write rows
+            resourcedata (Dict): Resource data
+            yearcol (Optional[Union[int,str]]): Year column for setting dataset year range. Defaults to None (don't set).
+            year_function (Optional[Callable[[Set[int],Dict],Optional[bool]]]): Year function to call for each row. Defaults to None.
+            quickcharts (Optional[Dict]): Dictionary containing optional keys: hashtag, values, cutdown and/or cutdownhashtags
+
+        Returns:
+            Tuple[bool, Dict]: (True if resource added, dictionary of results)
+        """
+        if yearcol is not None and year_function is not None:
+            raise HDXError('Supply either yearcol or year_function not both!')
+        retdict = dict()
+        if headers is None:
+            return False, retdict
+        rows = [Download.hxl_row(headers, hxltags, dict_form=True)]
+        years = set()
+        qc = {'cutdown': 0}
+        bites_disabled = [True, True, True]
+        if quickcharts is not None:
+            hashtag = quickcharts.get('hashtag')
+            if hashtag:
+                qc['column'] = next(key for key, value in hxltags.items() if value == hashtag)  # reverse dict lookup
+            else:
+                qc['column'] = None
+            cutdown = quickcharts.get('cutdown', 0)
+            if cutdown:
+                qc['cutdown'] = cutdown
+                cutdownhashtags = quickcharts.get('cutdownhashtags')
+                if cutdownhashtags is None:
+                    cutdownhashtags = hxltags.keys()
+                else:
+                    cutdownhashtags = [key for key, value in hxltags.items() if value in cutdownhashtags]
+                qc['headers'] = [x for x in headers if x in cutdownhashtags]
+                qc['rows'] = [Download.hxl_row(qc['headers'], hxltags, dict_form=True)]
+        for row in iterator:
+            if yearcol is not None:
+                year = row[yearcol]
+                if year:
+                    years.add(int(year))
+            elif year_function is not None:
+                ignore = year_function(years, row)
+                if ignore is True:
+                    continue
+            rows.append(row)
+            if quickcharts is not None:
+                if qc['column'] is None:
+                    if qc['cutdown']:
+                        qcrow = {x: row[x] for x in qc['headers']}
+                        qc['rows'].append(qcrow)
+                else:
+                    value = row[qc['column']]
+                    for i, lookup in enumerate(quickcharts['values']):
+                        if value == lookup:
+                            bites_disabled[i] = False
+                            if qc['cutdown']:
+                                qcrow = {x: row[x] for x in qc['headers']}
+                                qc['rows'].append(qcrow)
+
+        if len(rows) == 1:
+            logger.error('No data rows in %s!' % filename)
+            return False, retdict
+        if yearcol is not None or year_function is not None:
+            if len(years) == 0:
+                logger.error('No years in %s of %s!' % (yearcol, filename))
+                return False, retdict
+            else:
+                retdict['years'] = self.set_dataset_year_range(years)
+        self.generate_resource_from_rows(folder, filename, rows, resourcedata, headers=headers)
+        retdict['headers'] = headers
+        retdict['rows'] = rows
+        if quickcharts is not None:
+            retdict['bites_disabled'] = bites_disabled
+            if qc['cutdown']:
+                retdict['qcheaders'] = qc['headers']
+                retdict['qcrows'] = qc['rows']
+                if qc['cutdown'] == 2:
+                    qc_resourcedata = {'name': 'QuickCharts-%s' % resourcedata['name'],
+                                       'description': 'Cut down data for QuickCharts'}
+                    self.generate_resource_from_rows(folder, 'qc_%s' % filename, qc['rows'], qc_resourcedata,
+                                                     headers=qc['headers'])
+        return True, retdict
+
+    def download_and_generate_resource(self, downloader, url, hxltags, folder, filename, resourcedata,
+                                       header_insertions=None, row_function=None, yearcol=None, year_function=None,
+                                       quickcharts=None):
+        # type: (Download, str, Dict[str,str], str, str, Dict, Optional[List[Tuple[int,str]]], Optional[Callable[[List[str],Union[List,Dict]],Dict]], Optional[str], Optional[Callable[[Set[int],Union[List,Dict]],bool]], Optional[Dict]) -> Tuple[bool, Dict]
+        """Download url, write rows to csv and create resource, adding to it the dataset. The returned dictionary
+        will contain the headers in the key headers and the list of rows in the key rows.
+
+        Optionally, headers can be inserted at specific positions. This is achieved using the header_insertions
+        argument. If supplied, it is a list of tuples of the form (position, header) to be inserted. A function is
+        called for each row. If supplied, it takes as arguments: headers (prior to any insertions) and
+        row (which will be in dict or list form depending upon the dict_rows argument) and outputs a modified row.
+
+        The date of dataset can optionally be set by supplying a column in which the year is to be looked up or
+        a function to obtain the year from a row. The function should return False or None to keep the row or True
+        to ignore it. In either case, the list of years is returned in the key years of the returned dictionary.
+
+        If the parameter quickcharts is supplied then various QuickCharts related actions will occur depending upon the
+        keys given in the dictionary. If the keys: hashtag - the HXL hashtag to examine - and values - the 3 values to
+        look for in that column - are supplied, then a list of booleans indicating which QuickCharts bites should be
+        enabled will be returned in the key bites_disabled in the returned dictionary. If the key: cutdown is given,
+        if it is 1, then a separate cut down list is created containing only columns with HXL hashtags and
+        rows with desired values (if hashtag and values are supplied) for the purpose of driving QuickCharts.
+        It is returned in the key qcrows in the returned dictionary with the matching headers in qcheaders.
+        If cutdown is 2, then a resource is created using the cut down list. If the key cutdownhashtags is supplied,
+        then only the provided hashtags are used for cutting down otherwise the full list of hxl tags is used.
+
+        Args:
+            downloader (Download): Download object
+            url (str): URL to download
+            hxltags (Dict[str,str]): Header to HXL hashtag mapping
+            folder (str): Folder to which to write file containing rows
+            filename (str): Filename of file to write rows
+            resourcedata (Dict): Resource data
+            header_insertions (Optional[List[Tuple[int,str]]]): List of (position, header) to insert. Defaults to None.
+            row_function (Optional[Callable[[List[str],Union[List,Dict]],Union[List,Dict]]]): Function to call for each row. Defaults to None.
+            yearcol (Optional[str]): Year column for setting dataset year range. Defaults to None (don't set).
+            year_function (Optional[Callable[[Set[int],Dict],Optional[bool]]]): Year function to call for each row. Defaults to None.
+            quickcharts (Optional[Dict]): Dictionary containing optional keys: hashtag, values, cutdown and/or cutdownhashtags
+
+        Returns:
+            Tuple[bool, Dict]: (True if resource added, dictionary of results)
+        """
+        headers, iterator = downloader.get_tabular_rows(url, dict_form=True, header_insertions=header_insertions,
+                                                        row_function=row_function, format='csv')
+        return self.generate_resource_from_download(headers, iterator, hxltags, folder, filename, resourcedata,
+                                                    yearcol=yearcol, year_function=year_function,
+                                                    quickcharts=quickcharts)
